<?php
// This file is part of Moodle - https://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <https://www.gnu.org/licenses/>.

/**
 * Form for generating a data dump.
 *
 * @package     local_aatplugin
 * @copyright   2023 Joseph Larochelle
 * @license     https://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
namespace local_aatplugin\form;
use html_table;
use html_writer;
use moodleform;

defined('MOODLE_INTERNAL') || die();
require_once($CFG->libdir . "/formslib.php");

/**
 * Form for generating a data dump.
 */
class dump_form extends moodleform {
    /**
     * The courses where the current user is an instructor.
     *
     * @var array
     */
    private $courses;
    /**
     * Form definition method.
     */
    public function definition() {
        $mform = $this->_form; // Reference to the quickform object.

        // Initialize a table for course display.
        $this->initialize_course_table($mform);

        // Add submit button.
        $mform->addElement('submit', 'generate_dump', get_string('generatedump', 'local_aatplugin'));

        // Add a hidden element for storing file path.
        $mform->addElement('hidden', 'file_path');
        $mform->setType('file_path', PARAM_RAW); // Using PARAM_RAW as it's a file path.
    }

    /**
     * Initializes the HTML table for courses.
     *
     * @param MoodleQuickForm $mform The Moodle form instance
     */
    private function initialize_course_table($mform) {
        $table = new html_table();
        $table->head = [
                get_string('select', 'local_aatplugin'),
                get_string('courseid', 'local_aatplugin'),
                get_string('coursename', 'local_aatplugin'),
        ];
        $table->attributes['class'] = 'table table-striped table-bordered table-hover';

        foreach ($this->courses as $course) {
            $checkbox = '<input type="checkbox" name="courseid[]" value="' . $course->id . '"/>';
            $table->data[] = [$checkbox, $course->id, $course->fullname];
        }

        // Convert the table to HTML and add it as a static element.
        $tablehtml = html_writer::table($table);
        $mform->addElement('static', 'course_table', '', $tablehtml);
    }
    /**
     * Constructor for the form class.
     *
     * @param mixed $action Form action attribute.
     * @param mixed $customdata Any custom data needed for constructing the form.
     * @param string $method Form method attribute.
     * @param string $target Form target attribute.
     * @param mixed $attributes Form attributes.
     * @param bool $editable Whether the form is editable.
     */
    public function __construct($action = null, $customdata = null, $method = 'post', $target = '',
            $attributes = null, $editable = true) {
        if (isset($customdata['courses'])) {
            $this->courses = $customdata['courses'];
        }
        parent::__construct($action, $customdata, $method, $target, $attributes, $editable);
    }
}
