<?php
// This file is part of Moodle - https://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <https://www.gnu.org/licenses/>.

/**
 * Plugin version and other meta-data are defined here.
 *
 * @package     local_aatplugin
 * @copyright   2023 Joseph Larochelle
 * @license     https://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
require_once('../../config.php');
require_login();
/**
 * Identifies initial tables directly related to the course by checking field names.
 *
 * @param SimpleXMLElement $xml The XML structure containing the database schema.
 * @return array An associative array of initial table names with no parents.
 */
function findinitialtables(SimpleXMLElement $xml) {
    $initialtables = [];

    // Iterate through each table in the XML.
    foreach ($xml->TABLES->TABLE as $table) {
        // Check each field within the table for a 'course' or 'courseid' field.
        foreach ($table->FIELDS->FIELD as $field) {
            $fieldname = (string) $field['NAME'];
            if ($fieldname === 'course' || $fieldname === 'courseid') {
                $tablename = (string) $table['NAME'];
                $initialtables[$tablename] = null; // Mark as an initial table with no parent.
                break; // Stop checking fields once a match is found.
            }
        }
    }

    // Explicitly add the 'question' table as an initial table.
    $initialtables['question'] = null;

    return $initialtables;
}

/**
 * Finds all tables related to the initial set, considering foreign keys as relationships.
 *
 * @param SimpleXMLElement $xml The XML structure containing the database schema.
 * @param array $initialtables An associative array of known initial table names.
 * @return array An associative array mapping each related table name to its parent table and field.
 */
function findallrelatedtables(SimpleXMLElement $xml, array $initialtables) {
    $allrelatedtables = $initialtables; // Start with the initial tables.
    $processedtables = array_keys($initialtables); // Track processed tables to avoid repeats.

    do {
        $newlyaddedtables = [];

        // Iterate through each table in the XML.
        foreach ($xml->TABLES->TABLE as $table) {
            $tablename = (string) $table['NAME'];

            // Skip already processed tables.
            if (in_array($tablename, $processedtables)) {
                continue;
            }

            // Check if the table has foreign keys pointing to processed tables.
            foreach ($table->KEYS->KEY as $key) {
                $reftable = (string) $key['REFTABLE'];
                if (in_array($reftable, $processedtables)) {
                    $field = (string) $key['FIELDS'];
                    $allrelatedtables[$tablename] = [$reftable => $field]; // Associate with its reference.
                    $newlyaddedtables[] = $tablename; // Mark this table as newly added for this iteration.
                    break; // Stop checking other keys once a match is found.
                }
            }
        }

        // Merge newly added tables into the processed list for the next iteration.
        $processedtables = array_merge($processedtables, $newlyaddedtables);
    } while (!empty($newlyaddedtables)); // Continue until no new tables are found.

    return $allrelatedtables;
}
