<?php
// This file is part of Moodle - https://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <https://www.gnu.org/licenses/>.

/**
 * This file contains the database schema generator for the AAT plugin.
 *
 * @package     local_aatplugin
 * @copyright   2023 Joseph Larochelle
 * @license     https://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
require_once('../../config.php');
require_login();
// Constants for data types.
define('TYPE_INT', 'int');
define('TYPE_CHAR', 'char');
define('TYPE_NUMBER', 'number');
define('BIGINT', 'bigint');
define('CHAR_VARYING', 'character varying');
define('NUMERIC', 'numeric');
define('BIGSERIAL', 'BIGSERIAL PRIMARY KEY');
/**
 * Generate SQL CREATE TABLE statement for a given table structure. This
 * function is called from the 'index.php' file. To generate the schema of
 * the moodle database.
 *
 * @param SimpleXMLElement $table The table structure as a SimpleXMLElement.
 * @return string The SQL CREATE TABLE statement.
 */
function generatecreatetablesql($table) {
    $tablename = $table['NAME'];
    $fieldssql = generatefieldssql($table->FIELDS->FIELD);
    $sql = "CREATE TABLE public.mdl_$tablename (\n";
    $sql .= implode(",\n", $fieldssql) . "\n);\n\n";
    return $sql;
}

/**
 * Generate SQL for table fields.
 *
 * @param SimpleXMLElement[] $fields The fields as an array of SimpleXMLElement objects.
 * @return array An array of strings, each representing the SQL for a field.
 */
function generatefieldssql($fields) {
    $fieldssql = [];
    foreach ($fields as $field) {
        $fieldsql = generatefieldsql($field);
        $fieldssql[] = $fieldsql;
    }
    return $fieldssql;
}
/**
 * Generate SQL for a single field.
 *
 * @param SimpleXMLElement $field The field as a SimpleXMLElement object.
 * @return string The SQL for the field.
 */
function generatefieldsql($field) {
    $fieldname = $field['NAME'];
    $fieldtype = getfieldsqltype($field);
    $fieldsql = "\t$fieldname $fieldtype";

    // Handle the default value.
    if (!empty($field['DEFAULT'])) {
        $fieldsql .= " DEFAULT '" . $field['DEFAULT'] . "'";
    }

    // Handle NOT NULL constraint.
    if ($field['NOTNULL'] == 'true') {
        $fieldsql .= $field['TYPE'] == TYPE_CHAR && empty($field['DEFAULT']) ? " DEFAULT ''::character varying NOT NULL" :
                " NOT NULL";
    }

    // Handle the sequence for ID fields.
    if ($field['SEQUENCE'] == 'true' && $fieldname == 'id') {
        $fieldsql = "\tid " . BIGSERIAL;
    }
    return $fieldsql;
}

/**
 * Determine the SQL data type for a field.
 *
 * @param SimpleXMLElement $field The field as a SimpleXMLElement.
 * @return string The SQL data type for the field.
 */
function getfieldsqltype($field) {
    switch ($field['TYPE']) {
        case TYPE_INT:
            return BIGINT;
        case TYPE_CHAR:
            return CHAR_VARYING;
        case TYPE_NUMBER:
            return NUMERIC . "(" . $field['LENGTH'] . ", " . $field['DECIMALS'] . ")";
        default:
            return $field['TYPE'];
    }
}


