<?php
// This file is part of Moodle - https://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <https://www.gnu.org/licenses/>.

/**
 * This file contains the data dump generator for the AAT plugin.
 *
 * @package     local_aatplugin
 * @copyright   2023 Joseph Larochelle
 * @license     https://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
require_once('../../config.php');
require_login();
require_once('installXMLMerger.php');
require_once('courseTableRelationsMapper.php');
require_once('databaseSchemaGenerator.php');
global $DB;
/**
 * Generate a SQL dump of the data related to the courses.
 * @param array $courseids
 * @param string $filepath
 * @return void
 * @throws Exception
 */
function generate_sql_dump(array $courseids, string $filepath) {
    // Extract all the keys from the array of courseids.
    $courseids = array_values($courseids);
    global $DB, $CFG;
    // Initialize the sql dump.
    // Add a comment to the sql dump file with the moodle version.
    $sqldump = "-- AAT plugin SQL dump\n";
    $sqldump .= "-- {$CFG->release}\n\n";

    // Check if the combined_XML_files.xml file exists.
    if (!file_exists($CFG->dataroot . '/AATplugin/combined_XML_files.xml')) {
        // Create the file and folder.
        if (!mkdir($concurrentdirectory = $CFG->dataroot . '/AATplugin') && !is_dir($concurrentdirectory)) {
            throw new RuntimeException(sprintf('Directory "%s" was not created', $concurrentdirectory));
        }
        // If it does not exist, create it.
        combineXMLFiles($CFG->dirroot );
    }
    // Check if the schema file exists.
    if (!file_exists($CFG->dataroot . '/AATplugin/database_schema.sql')) {
        // If it does not exist, create it.
        // Load and parse the XML file.
        $xmlfilepath = $CFG->dataroot . '/AATplugin/combined_XML_files.xml';
        $xml = simplexml_load_file($xmlfilepath);
        // Open a file to write the SQL statements.
        $sqlfile = fopen($CFG->dataroot ."/AATplugin/database_schema.sql", "w");
        // Generate SQL for each table and write to the file.
        foreach ($xml->TABLES->TABLE as $table) {
            $sqlstatement = generateCreateTableSQL($table);
            fwrite($sqlfile, $sqlstatement);
        }
        // Close the file.
        fclose($sqlfile);
    }
    // Add the schema file to the sql dump.
    $schema = file_get_contents($CFG->dataroot . '/AATplugin/database_schema.sql');
    $sqldump .= $schema;
    // Check to see if the related_tables.txt file exists.
    if (!file_exists($CFG->dataroot .'/AATplugin/related_tables.txt')) {
        // If it does not exist, create it.
        // Path to your XML file.
        $xmlfilepath = $CFG->dataroot . '/AATplugin/combined_XML_files.xml';
        $xml = simplexml_load_file($xmlfilepath);

        $initialtables = findInitialTables($xml);
        $relatedtableswithparent = findAllRelatedTables($xml, $initialtables);

        // Print the related tables with their direct parent.
        $serializedoutput = serialize($relatedtableswithparent);
        file_put_contents($CFG->dataroot .'/AATplugin/related_tables.txt', $serializedoutput);
    }
    $serializedtables = file_get_contents($CFG->dataroot .'/AATplugin/related_tables.txt');
    $relatedtables = unserialize($serializedtables, ["allowed_classes" => false]);

    // Iterate through the array and collect all tables that do not have a subtable.
    // These tables are direct children of the course table.
    $coursetables = [];
    foreach ($relatedtables as $table => $subtable) {
        // If the table has no subtable, add it to the array of course tables.
        if ($subtable == null) {
            $coursetables[$table] = [];
            // Remove it from the array of related tables.
            unset($relatedtables[$table]);
        }
    }
    unset($coursetables['question']);
    // All the tables in $coursetables are direct children of the course table.
    // We iterate through the entire array and add all of the records that are related to the courses.
    // Listed in the $courseids array.
    foreach ($coursetables as $table => $ids) {
        // Prepend the 'mdl_' prefix to the table name.
        $prefixedtablename = $CFG->prefix . $table;
        // Get the columns of the table.
        try {
            $columns = $DB->get_columns($table);
            $field = array_key_exists('courseid', $columns) ? 'courseid' : 'course';
            // Fetch records related to the course.
            $records = get_table_records($table, $field, $courseids);
            $sqldump .= create_sql_statements($records, $table, $coursetables);
        } catch (Exception $e) {
            echo 'Caught exception: ' . $e->getMessage() . "error1\n";
            echo '<br>';
        }
    }

    // Fetch all the rows in the mdl_course table matching the courseids.
    $coursesrecords = get_table_records('course', 'id', $courseids);
    // Add those records to the sql dump.
    $sqldump .= create_sql_statements($coursesrecords, 'course', $coursetables);

    // Add all rows from the mdl_config table to the sql dump.
    $configsrecords = $DB->get_records('config');
    $sqldump .= create_sql_statements($configsrecords, 'config', $coursetables);

    // Get all the distinct userids from the logstore_standard_log table where courseid is in the array of courseids.
    $logstorestandardlogrecords = $DB->get_records_list('logstore_standard_log', 'courseid', $courseids);
    $userids = [];
    foreach ($logstorestandardlogrecords as $logstorestandardlogrecord) {
        if (!empty($logstorestandardlogrecord->userid)) {
            $userids[$logstorestandardlogrecord->userid] = true;
        }
        if (!empty($logstorestandardlogrecord->relateduserid)) {
            $userids[$logstorestandardlogrecord->relateduserid] = true;
        }
    }
    // Get all the records from the user table where the userid is in the array of userids.
    $usersrecords = $DB->get_records_list('user', 'id', array_keys($userids));
    $sqldump .= create_sql_statements($usersrecords, 'user', $coursetables);
    // Get all the records from the question table where createdby or modifiedby is in the array of userids.
    $questionsrecords = $DB->get_records_list('question', 'createdby', array_keys($userids));

    $sqldump .= create_sql_statements($questionsrecords, 'question', $coursetables);
    $questionsrecords = $DB->get_records_list('question', 'modifiedby', array_keys($userids));
    $sqldump .= create_sql_statements($questionsrecords, 'question', $coursetables);
    // While there are tables in the array of tables to be added to the sql dump.
    for ($i = 0; $i < 3; $i++) {
        $errors = 0;
        // Iterate through the array of table in relatedtables.
        foreach ($relatedtables as $table => $subtable) {
            // Check if the key of $subtable is in the array of keys of $coursetables.
            if (array_key_exists(key($subtable), $coursetables)) {
                // Prepend the 'mdl_' prefix to the table name.
                $prefixedtablename = $CFG->prefix . $table;
                // Get the value of the key of $subtable.
                $field = $subtable[key($subtable)];
                try {
                    // Fetch records related to the course.
                    $records = get_table_records($table, $field, $coursetables[key($subtable)]);
                } catch (Exception $e) {
                    if ($errors == 0) {
                        echo 'There was an error creating the dump file. The file can be used
                         but data from the following table(s) may be missing:<br> ';
                        $errors = 1;
                    }
                    // Print the table name.
                    echo '<br>';
                    echo $table;
                    echo '<br>';
                }
                $sqldump .= create_sql_statements($records, $table, $coursetables);
                // Remove the table from the array of related tables.
                unset($relatedtables[$table]);
            }
        }
    }
    // This section is used to add all the tables with data that is not directly related to courses but is used in AAT.
    // Because of the lack of relationship they need to be hardcoded into the plugin.
    // Book chapters.
    $bookchaptersrecords = $DB->get_records_list('book_chapters', 'bookid', $coursetables['book']);
    $sqldump .= create_sql_statements($bookchaptersrecords, 'course', $coursetables);
    // Return all the records from the course_categories table.
    $coursecategoriesrecords = $DB->get_records('course_categories');
    $sqldump .= create_sql_statements($coursecategoriesrecords, 'course_categories', $coursetables);
    // Return the records from the course_modules_completion table.
    $coursemodulescompletionrecords = $DB->get_records_list
    ('course_modules_completion', 'coursemoduleid', $coursetables['course_modules']);
    $sqldump .= create_sql_statements($coursemodulescompletionrecords, 'course_modules_completion', $coursetables);
    // Return the records from the files table.
    $filesrecords = $DB->get_records_list('files', 'userid', $coursetables['user']);
    $sqldump .= create_sql_statements($filesrecords, 'files', $coursetables);
    // Add the records from repository_instances table.
    $repositoryinstancesrecords = $DB->get_records_list('repository_instances', 'userid', $coursetables['user']);
    $sqldump .= create_sql_statements($repositoryinstancesrecords, 'repository_instances', $coursetables);
    // Add the records from file_references table.
    if (!empty($coursetables['repository_instances'])) {
        $filereferencesrecords = $DB->get_records_list('file_references', 'repositoryid', $coursetables['repository_instances']);
        $sqldump .= create_sql_statements($filereferencesrecords, 'file_references', $coursetables);
    }
    // Add the records from forum_read table.
    $forumreadrecords = $DB->get_records_list('forum_read', 'forumid', $coursetables['forum']);
    $sqldump .= create_sql_statements($forumreadrecords, 'forum_read', $coursetables);
    // Add the records from forum_track_prefs table.
    $forumtrackprefsrecords = $DB->get_records_list('forum_track_prefs', 'forumid', $coursetables['forum']);
    $sqldump .= create_sql_statements($forumtrackprefsrecords, 'forum_track_prefs', $coursetables);
    // Add the records from lti_submissions table.
    $ltisubmissionsrecords = $DB->get_records_list('lti_submission', 'ltiid', $coursetables['lti']);
    $sqldump .= create_sql_statements($ltisubmissionsrecords, 'lti_submission', $coursetables);
    // Add records from wiki_synonyms table.
    if (!empty($coursetables['wiki_subwikis'])) {
        $wikisynonymsrecords = $DB->get_records_list('wiki_synonyms', 'subwikiid', $coursetables['wiki_subwikis']);
        $sqldump .= create_sql_statements($wikisynonymsrecords, 'wiki_synonyms', $coursetables);
    }
    // Add all entries from the role table.
    $rolesrecords = $DB->get_records('role');
    $sqldump .= create_sql_statements($rolesrecords, 'role', $coursetables);
    // Get all the records from the role_assignments table where the userid is in the array of userids.
    $roleassignmentsrecords = $DB->get_records_list('role_assignments', 'userid', array_keys($userids));
    $sqldump .= create_sql_statements($roleassignmentsrecords, 'role_assignments', $coursetables);

    // Load the public key.
    $publickey = openssl_get_publickey(file_get_contents($CFG->dirroot . '/local/aatplugin/public_key.pem'));

    if (!$publickey) {
        throw new Exception('Failed to load public key');
    }

    // Define the key size.
    $keydetails = openssl_pkey_get_details($publickey);
    $keysize = $keydetails['bits'] / 8 - 11; // Key size in bytes.
    // Break the data into chunks and encrypt each one.
    $chunks = str_split($sqldump, $keysize);
    $encrypted = '';

    foreach ($chunks as $chunk) {
        $encryptedchunk = '';
        if (!openssl_public_encrypt($chunk, $encryptedchunk, $publickey)) {
            throw new Exception('Failed to encrypt data');
        }
        $encrypted .= $encryptedchunk;
    }

    // Write the encrypted data to the file.
    file_put_contents($filepath, $encrypted);
}
/**
 * Get records from a table based on a field and values.
 * @param string $table
 * @param string $field
 * @param array $values
 * @return array
 */
function get_table_records($table, $field, $values) {
    global $DB;
    return $DB->get_records_list($table, $field, $values);
}
/**
 * Create SQL statements for a set of records.
 *
 * @param array $records
 * @param string $table
 * @param array $coursetables
 * @return string
 */
function create_sql_statements($records, $table, &$coursetables) {
    // Check to see if the records are empty.
    if (!empty($records)) {
        global $CFG;
        $prefixedtablename = $CFG->prefix . $table;
        $sqldump = "";
        foreach ($records as $record) {
            // Get the value of the id field.
            $id = $record->id;
            // Add the id to $courseTables.
            $coursetables[$table][] = $id;

            $columns = array_keys(get_object_vars($record));
            $valuestrings = [];

            foreach ($record as $columnname => $value) {
                $escapedvalue = ($value === null) ? 'NULL' : "E'" . addslashes($value) . "'";
                $valuestrings[] = $escapedvalue;
            }

            $sqldump .= "INSERT INTO " . $prefixedtablename . "(" . implode(", ", $columns) . ") VALUES (" .
                    implode(",", $valuestrings) .
                    ");\n ";
        }
        return $sqldump;
    }
}
