<?php
// This file is part of Moodle - https://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <https://www.gnu.org/licenses/>.

/**
 * Main plugin file
 * @package     local_aatplugin
 * @copyright   2023 Joseph Larochelle <larochellejo@hotmail.com>
 * @license     https://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
use local_aatplugin\form\dump_form;
require_once('../../config.php');
require_once("$CFG->libdir/formslib.php");
require_once($CFG->dirroot . '/local/aatplugin/lib.php');
require_once($CFG->dirroot . '/local/aatplugin/classes/form/dump_form.php');
require_once($CFG->dirroot . '/local/aatplugin/datadump.php');

// Set up the page context and properties.
global $PAGE, $SITE;
$context = context_system::instance();
$PAGE->set_context($context);
$PAGE->set_url(new moodle_url('/local/aatplugin/index.php'));
$PAGE->set_pagelayout('standard');
$PAGE->set_title($SITE->fullname);
$PAGE->set_heading(get_string('pluginname', 'local_aatplugin'));
require_login();

// Output the header and plugin information.
echo $OUTPUT->header();
echo get_string('infomessage', 'local_aatplugin');
echo '<hr>'; // Visual separation.

// Fetch user's courses and prepare form.
$courses = fetch_user_courses();
$dumpform = new dump_form(null, ['courses' => $courses]);
process_form_submission($dumpform);

// Always display the link to AAT website at the bottom.
echo '<hr>';
display_instructions();
echo $OUTPUT->footer();


/**
 * Fetches the courses where the current user is an instructor.
 *
 * @return array An array of course objects.
 */
function fetch_user_courses() {
    global $DB, $USER;
    $sql = "SELECT c.id, c.fullname
            FROM {course} c
            JOIN {context} ctx ON c.id = ctx.instanceid
            JOIN {role_assignments} ra ON ra.contextid = ctx.id
            WHERE ra.userid = :userid AND ra.roleid = 3 AND ctx.contextlevel = 50";

    return $DB->get_records_sql($sql, ['userid' => $USER->id]);
}

/**
 * Handles the form submission, generating an SQL dump if necessary.
 *
 * @param dump_form $dumpform The form instance.
 */
function process_form_submission($dumpform) {
    if ($data = $dumpform->get_data()) {
        $selectedcourses = optional_param_array('courseid', [], PARAM_INT);
        if (empty($selectedcourses)) {
            display_selection_error($dumpform);
            return;
        }
        generate_and_download_sql_dump($selectedcourses);
    } else {
        echo '<br><br>';
        $dumpform->display();
    }
}

/**
 * Displays an error message when no courses are selected and re-displays the form.
 *
 * @param dump_form $dumpform The form instance.
 */
function display_selection_error($dumpform) {
    echo '<br>';
    echo html_writer::tag('h5', "You have not selected any courses, please select at least one course");
    echo '<br>';
    $dumpform->display();
}

/**
 * Generates the SQL dump for selected courses and provides a download link.
 *
 * @param array $selectedcourses The IDs of the selected courses.
 */
function generate_and_download_sql_dump($selectedcourses) {
    global $CFG;

    $courseidsstring = implode('_', $selectedcourses);
    $filename = "course_{$courseidsstring}_dump_plugin.sql";
    $filepath = $CFG->dataroot . "/" . $filename;
    generate_sql_dump($selectedcourses, $filepath);

    // Display the download link.
    echo '<br>';
    echo html_writer::tag('h4', "Your file was created, click the link below to download it");
    echo '<br>';
    echo html_writer::start_tag('div', ['style' => 'display: flex; justify-content: center; align-items: center;']);
    echo html_writer::tag('a', 'Download SQL Dump', [
            'href' => new moodle_url('/local/AATplugin/downloadfile.php', ['file' => $filepath]),
            'class' => 'btn btn-primary' ,
    ]);
    echo html_writer::end_tag('div');

    // Add a link to go back to the course selection page.
    echo '<br>';
    echo html_writer::start_tag('div', ['style' => 'display: flex; justify-content: center; align-items: center;']);
    echo html_writer::tag('a', 'Go back to course selection', [
            'href' => new moodle_url('/local/AATplugin/index.php'),
            'class' => 'btn btn-secondary' ,
    ]);
    echo html_writer::end_tag('div');
}

/**
 * Displays the link to the AAT website.
 */
function display_instructions() {
    echo '<h5>Instructions:</h5><br>';
    // Create a list of instructions.
    echo html_writer::tag('p', "1. Select the courses you want to export and click the 'Generate SQL Dump' button");
    echo html_writer::tag('p', "2. Once the file is generated, click the 'Download SQL Dump' link to download the file
                            to your computer. Take note of the file location.");
    echo html_writer::tag('p', "3. Head over to the AAT website (link provided below) and log in using your credentials");
    echo html_writer::tag('p', "4. Within the AAT website, navigate to the 'Data Source' tab and click on the 'Upload' button");
    echo html_writer::tag('p', "5. Select 'Postgres' as the vendor, type in a name for the database, and select the file that was
                           downloaded in step 2. Click 'Save' to complete the process.");
    echo html_writer::tag('p', "You now should have access to the data within the AAT tool.");
    echo html_writer::tag('a', "https://www.academicanalytics.ca/", [
            'href' => 'https://www.academicanalytics.ca/',
            'target' => '_blank' ,
    ]);
}
